package Source::Package;

use File::Path qw(make_path remove_tree);
use File::Basename;

use strict;
use warnings;

our $VERSION=0.02;

sub new {
    my $proto=shift;
    my $class=ref($proto) || $proto;
    my $self = {
	-cachedir=>'in',
	-epoch => undef,
	-version => 0,
	-release => 0,
	@_,
    };
    bless($self,$class);
}

sub version {return $_[0]->{-version}}
sub release {return $_[0]->{-release}}
sub epoch {return $_[0]->{-epoch}}
sub name {return $_[0]->{-name}}
sub url {return $_[0]->{-url}}
sub file {
    my ($self)=@_;
    my $file=$self->{-file};
    return $file if defined $file;
    return $self->__download if $self->{-url};
    return $self->{-repo}->download($self) if $self->{-repo};
    return;
}

### TODO check url?
sub __download {
    my ($self)=@_;
    my $file=$self->{-file};
    return $file if defined $file and -e $file and -s $file;
    my $url=$self->url;
    my $filename=basename($url);
    $filename=~s/\?.*$//;
    my $cachedir=$self->__cachedir;
    make_path($cachedir);
    $file="$cachedir/$filename";
    return $file if -e $file and -s $file;
    unlink $file;
    `wget -nv -O $file $url`;
    unlink $file if -z $file;
    die "download failed" unless -e $file;
    $self->{-file}=$file;
    return $file;
}

# for download only
sub __cachedir {
    my ($self)=@_;
    my $cachedir=$self->{-cachedir};
    return $cachedir if defined $cachedir;
    return 'in';
}

1;
