package Source::Package::Comparator;

use strict;
use warnings;
use Source::Package::Comparators;

our $VERSION=0.001;
our $verbose;

our $default_epoch_comparator=  \&Source::Package::Comparators::null_epoch_comparator;
our $default_version_comparator=\&Source::Package::Comparators::pure_rpm_version_comparator;
our $default_release_comparator=\&Source::Package::Comparators::pure_rpm_release_comparator;

sub new {
    my $class = shift;
    my $self = {
	-epoch_comparator   => $default_epoch_comparator,
	-version_comparator => $default_version_comparator,
	-release_comparator => $default_release_comparator,
	@_
    };
    bless $self, $class;
    return $self;
}

#Binary "cmp" returns -1, 0, or 1 depending on whether the left argument
#is stringwise less than, equal to, or greater than the right argument.
sub cmp {
    my ($self,$origin, $dest) =@_;
    return 1 unless $dest;
    return -1 unless $origin;
    my $cmp =&{$self->{-epoch_comparator}}($origin, $dest);
    if ($cmp!=0) {
	print STDERR "Package::Comparator: epoch:". $origin->epoch.' cmp '.$dest->epoch." = $cmp\n" if $verbose;
	return $cmp;
    }
    $cmp =&{$self->{-version_comparator}}($origin, $dest);
    if ($cmp!=0) {
	print STDERR "Package::Comparator: version:". $origin->version.' cmp '.$dest->version." = $cmp\n" if $verbose;
	return $cmp;
    }
    return &{$self->{-release_comparator}}($origin, $dest);
}

1;
