package Source::Package::Comparators::Raw;

use strict;
use warnings;
use Carp;
use version;
use RPM qw/evrcmp vercmp/;

our $VERSION=0.001;
our $verbose=0;

###### PROTECTED #############################################

# used in ALTLinuxImport
sub _rpm_plus_perl_version_cmp {
    my ($originname, $destname, $originversion, $destversion)=@_;
    Carp::confess 'Oops!: undefined versions' unless defined $destversion and defined $originversion;
    $originname='(undef)' if not defined $originname;
    if ($destname =~/^perl-/) {
	my $compare_result;
	eval {
	    $compare_result = &perl_version_cmp($originversion, $destversion);
	    };
	die "_rpm_plus_perl_version_cmp: $originname, $destname, $originversion, $destversion: $@.\n" if $@;
	return $compare_result;
    }
    return vercmp($originversion, 0, $destversion, 0);
}

sub perl_version_cmp {
    my ($originversion,$destversion)=@_;
    Carp::confess 'incorrect call: 3-rd,4-th args found!' if $_[2] or $_[3];
    my $cmp=&__raw_perl_version_cmp($originversion,$destversion);
    $cmp=0 if $cmp>0 and $destversion.'.0' eq $originversion; # like, 1.4 equal 1.4.0
    $cmp=0 if $cmp<0 and $originversion.'.0' eq $destversion; # like, 1.4 equal 1.4.0
    return $cmp;
}

###### PRIVATE #############################################

sub __raw_perl_version_cmp {
    my ($ver1, $ver2)=@_;
    my ($v1s,$tail1)=&__raw_perl_ver_trim($ver1);
    my ($v2s,$tail2)=&__raw_perl_ver_trim($ver2);
    my $v1=version->parse($v1s);
    my $v2=version->parse($v2s);
    if ($v1 == $v2) {
	if (not $tail1 and not $tail2) {
	    return 0;
	} else {
	    #warn "tails found in $v1s $v2s\n";
	    if ($tail2 and not $tail1 and $tail2=~/[-\._]?(alpha|beta)\d$/) {
		# 1.0 > 1.0beta3
		return 1;
	    } else {
		return RPM::evrcmp($tail1, $tail2);
	    }
	}
    }

    #warn "methods differ! $v1s, $v2s\n" if (evrcmp($s1, $s2) < 0) xor ($v1 < $v2);
    return $v1 cmp $v2;
}

sub __raw_perl_ver_trim {
    my $vers=shift;
    $vers=~s/^V/v/;
    $vers=~s/^(v?[\d\.]+)([^\d\.].*)?$/$1/;
    my $tail=$2;
    $tail||='';
    return ($vers,$tail);
}

1;
